/***************************************************************************
**
** Copyright (C) 2013 Jolla Ltd.
** Contact: Aaron Kennedy <aaron.kennedy@jollamobile.com>
**
** This file is part of lipstick.
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License version 2.1 as published by the Free Software Foundation
** and appearing in the file LICENSE.LGPL included in the packaging
** of this file.
**
****************************************************************************/

#ifndef LIPSTICKCOMPOSITORWINDOW_H
#define LIPSTICKCOMPOSITORWINDOW_H

#include <QWaylandQuickItem>
#include <QWaylandBufferRef>

#include <QWaylandWlShellSurface>
#include "lipstickglobal.h"

#include <QPointer>

QT_BEGIN_NAMESPACE

namespace QtWayland {
    class ExtendedSurface;
}

class QWaylandXdgSurfaceV5;

QT_END_NAMESPACE

class LipstickCompositor;
class LipstickCompositorWindowHwcNode;

class WindowPropertyMap;

class LIPSTICK_EXPORT LipstickCompositorWindow : public QWaylandQuickItem
{
    Q_OBJECT

    Q_PROPERTY(int windowId READ windowId CONSTANT)
    Q_PROPERTY(bool isInProcess READ isInProcess CONSTANT)
    Q_PROPERTY(bool isAlien READ isAlien CONSTANT)

    Q_PROPERTY(bool delayRemove READ delayRemove WRITE setDelayRemove NOTIFY delayRemoveChanged)
    Q_PROPERTY(QVariant userData READ userData WRITE setUserData NOTIFY userDataChanged)

    Q_PROPERTY(QString category READ category CONSTANT)
    Q_PROPERTY(QString title READ title NOTIFY titleChanged)
    Q_PROPERTY(qint64 processId READ processId CONSTANT)
    Q_PROPERTY(qint16 windowFlags READ windowFlags NOTIFY windowFlagsChanged)
    Q_PROPERTY(QString className READ className NOTIFY classNameChanged)

    Q_PROPERTY(bool focusOnTouch READ focusOnTouch WRITE setFocusOnTouch NOTIFY focusOnTouchChanged)

    Q_PROPERTY(WindowPropertyMap *properties READ windowProperties CONSTANT)

public:
    LipstickCompositorWindow(int windowId, const QString &, QWaylandSurface *surface = nullptr);
    LipstickCompositorWindow(int windowId, const QString &, QWaylandWlShellSurface *wlSurface);
    LipstickCompositorWindow(int windowId, const QString &, QWaylandXdgSurfaceV5 *xdgSurface);
    ~LipstickCompositorWindow();

    QVariant userData() const;
    void setUserData(QVariant);

    int windowId() const;
    qint64 processId() const;

    bool delayRemove() const;
    void setDelayRemove(bool);

    bool isTransient() const;

    QString category() const;
    virtual QString title() const;
    virtual bool isInProcess() const;

    QString className() const;

    qint16 windowFlags();

    bool eventFilter(QObject *object, QEvent *event);

    Q_INVOKABLE void terminateProcess(int killTimeout);

    QSGNode *updatePaintNode(QSGNode *old, UpdatePaintNodeData *);

    bool focusOnTouch() const;
    void setFocusOnTouch(bool focusOnTouch);

    bool isAlien() const;

    WindowPropertyMap *windowProperties();
    QVariant windowProperty(const QString &key);

    QtWayland::ExtendedSurface *extendedSurface();

    Q_INVOKABLE void resize(const QSize &size);
    Q_INVOKABLE void ping();
    Q_INVOKABLE void close();
    Q_INVOKABLE void closePopup();

protected:
    void setTitle(const QString &title);

    void itemChange(ItemChange change, const ItemChangeData &data);

    virtual bool event(QEvent *);
    virtual void mousePressEvent(QMouseEvent *event);
    virtual void mouseMoveEvent(QMouseEvent *event);
    virtual void mouseReleaseEvent(QMouseEvent *event);
    virtual void wheelEvent(QWheelEvent *event);
    virtual void touchEvent(QTouchEvent *event);

signals:
    void userDataChanged();
    void titleChanged();
    void classNameChanged();
    void delayRemoveChanged();
    void focusOnTouchChanged();
    void windowFlagsChanged();
    void pong();

private slots:
    void handleTouchCancel();
    void killProcess();

private:
    friend class LipstickCompositor;
    friend class WindowModel;
    friend class WindowPixmapItem;

    void imageAddref(QQuickItem *item);
    void imageRelease(QQuickItem *item);
    void onSync();

    bool canRemove() const;
    void tryRemove();
    void handleTouchEvent(QTouchEvent *e);

    void setExtendedSurface(QtWayland::ExtendedSurface *extSurface);
    QString m_title;
    QString m_category;
    QVariant m_data;

    QVector<QQuickItem *> m_refs;
    QPointer<QtWayland::ExtendedSurface> m_extSurface;
    QPointer<QWaylandWlShellSurface> m_wlShellSurface;
    QPointer<QWaylandXdgSurfaceV5> m_xdgSurface;
    QScopedPointer<WindowPropertyMap> m_windowProperties;

    LipstickCompositor * const m_compositor;
    const int m_windowId;
    uint m_pingSerial = 0;
    bool m_isAlien : 1;
    bool m_delayRemove : 1;
    bool m_windowClosed : 1;
    bool m_removePosted : 1;
    bool m_interceptingTouch : 1;
    bool m_mapped : 1;
    bool m_noHardwareComposition: 1;
    bool m_focusOnTouch : 1;
    bool m_hasVisibleReferences : 1;
    bool m_transient : 1;
};

#endif // LIPSTICKCOMPOSITORWINDOW_H
